package org.jvnet.hifaces.model.paging;

import java.util.Collections;

/**
 * Paging data model.
 * 
 * @author valikov
 * 
 * @param <T>
 *            Type of the entry.
 */
public class PagingDataModel<T> extends javax.faces.model.DataModel {

	/**
	 * Size of the page.
	 */
	private final int pageSize;

	/**
	 * Page deliverer.
	 */
	private final DataPageable<T> dataPageable;

	/**
	 * Current row index.
	 */
	private int rowIndex;

	/**
	 * Current page.
	 */
	private DataPage page;

	/**
	 * Constructs a paging model for given page deliverer.
	 * 
	 * @param pageSize
	 *            size of the page.
	 * @param dataPageable
	 *            page deliverer.
	 */
	public PagingDataModel(int pageSize, DataPageable<T> dataPageable) {
		this.pageSize = pageSize;
		this.dataPageable = dataPageable;
	}

	/**
	 * Not supported.
	 */
	public void setWrappedData(Object data) {
		throw new UnsupportedOperationException();
	}

	/**
	 * {@inheritDoc}
	 */
	public int getRowIndex() {
		return rowIndex;
	}

	/**
	 * {@inheritDoc}
	 */
	public void setRowIndex(int rowIndex) {

		this.rowIndex = rowIndex;
	}

	/**
	 * {@inheritDoc}
	 */
	public int getRowCount() {
		final DataPage page = getPage();
		if (page == null) {
			return 0;
		} else {
			return page.getCount();
		}
	}

	/**
	 * {@inheritDoc}
	 */
	protected DataPage getPage() {
		if (page != null) {
			return page;
		} else {

			final int rowIndex = getRowIndex();
			final int startRow = rowIndex == -1 ? 0 : rowIndex;
			// invoke method on enclosing class
			page = getPage(startRow, pageSize);
			return page;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public Object getRowData() {
		if (rowIndex < 0) {
			throw new IllegalArgumentException("Invalid rowIndex.");
		} else {

			if (page == null) {
				page = getPage(rowIndex, pageSize);
			}

			final int datasetSize = page.getCount();
			int startRow = page.getStart();
			final int nRows = page.getData().size();
			final int endRow = startRow + nRows;

			if (rowIndex >= datasetSize) {
				rowIndex = 0;
			}

			if (rowIndex < startRow) {
				page = getPage(rowIndex, pageSize);
				startRow = page.getStart();
			} else if (rowIndex >= endRow) {
				page = getPage(rowIndex, pageSize);
				startRow = page.getStart();
			}

			final Object object = page.getData().get(rowIndex - startRow);

			return object;
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public Object getWrappedData() {
		if (page == null) {
			return Collections.EMPTY_LIST;
		} else {
			return page.getData();
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public boolean isRowAvailable() {
		final DataPage page = getPage();
		if (page == null) {
			return false;
		} else {
			final int rowIndex = getRowIndex();
			if (rowIndex < 0) {
				return false;
			} else if (rowIndex >= page.getCount()) {
				return false;
			} else {
				return true;
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public DataPage getPage(int start, int pageSize) {
		return dataPageable.getPage(start, pageSize);
	}
}
