package org.jvnet.hifaces.util;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.faces.FactoryFinder;
import javax.faces.application.Application;
import javax.faces.application.ApplicationFactory;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIViewRoot;
import javax.faces.component.html.HtmlMessage;
import javax.faces.context.FacesContext;

import org.apache.commons.lang.Validate;

public class MessageUtils {

	/**
	 * Default detail message suffix.
	 */
	public static final String DETAIL_SUFFIX = "_detail";

	private MessageUtils() {
	}

	/**
	 * Creates a message component for the given client id.
	 * 
	 * @param forId
	 *            target client id.
	 * @return Message component.
	 */
	public static HtmlMessage createMessage(String forId) {
		final HtmlMessage message = new HtmlMessage();
		IdUtils.assignId(message);
		message.setFor(forId);
		return message;
	}

	public static void addMessage(String forClientId, String messageId) {
		addMessage(FacesContext.getCurrentInstance(), forClientId, messageId);
	}

	public static void addMessage(String forClientId, String messageId,
			Object... params) {
		addMessage(FacesContext.getCurrentInstance(), forClientId, messageId,
				params);
	}

	public static void addMessage(String forClientId,
			FacesMessage.Severity severity, String messageId) {
		addMessage(FacesContext.getCurrentInstance(), forClientId, severity,
				messageId);
	}

	public static void addMessage(String forClientId,
			FacesMessage.Severity severity, String messageId, Object... params) {
		addMessage(FacesContext.getCurrentInstance(), forClientId, severity,
				messageId, params);
	}

	public static void addMessage(FacesContext facesContext,
			String forClientId, String messageId) {
		addMessage(facesContext, forClientId, FacesMessage.SEVERITY_INFO,
				messageId, null);
	}

	public static void addMessage(FacesContext facesContext,
			String forClientId, String messageId, Object... params) {
		addMessage(facesContext, forClientId, FacesMessage.SEVERITY_INFO,
				messageId, params);
	}

	public static void addMessage(FacesContext facesContext,
			String forClientId, FacesMessage.Severity severity, String messageId) {
		addMessage(facesContext, forClientId, severity, messageId, null);
	}

	public static void addMessage(FacesContext facesContext,
			String forClientId, FacesMessage.Severity severity,
			String messageId, Object... params) {
		final FacesMessage message = getMessage(facesContext, severity,
				messageId, params);
		facesContext.addMessage(forClientId, message);
	}

	public static FacesMessage getMessage(String messageId) {
		return getMessage(FacesContext.getCurrentInstance(),
				FacesMessage.SEVERITY_INFO, messageId, null);
	}

	public static FacesMessage getMessage(String messageId, Object... params) {
		return getMessage(FacesContext.getCurrentInstance(),
				FacesMessage.SEVERITY_INFO, messageId, params);
	}

	public static FacesMessage getMessage(FacesMessage.Severity severity,
			String messageId) {
		return getMessage(FacesContext.getCurrentInstance(), severity,
				messageId, null);
	}

	public static FacesMessage getMessage(FacesMessage.Severity severity,
			String messageId, Object... params) {
		return getMessage(FacesContext.getCurrentInstance(), severity,
				messageId, params);
	}

	public static FacesMessage getMessage(FacesContext context, String messageId) {
		return getMessage(context, FacesMessage.SEVERITY_INFO, messageId, null);
	}

	public static FacesMessage getMessage(FacesContext context,
			String messageId, Object... params) {
		return getMessage(context, FacesMessage.SEVERITY_INFO, messageId,
				params);
	}

	public static FacesMessage getMessage(FacesContext context,
			FacesMessage.Severity severity, String messageId) {
		return getMessage(context, severity, messageId, null);
	}

	public static FacesMessage getMessage(FacesContext context,
			FacesMessage.Severity severity, String messageId, Object... params) {
		Validate.notNull(severity, "Severity must not be null.");
		Validate.notNull(context, "Context must not be null.");
		Validate.notNull(messageId, "Message id must not be null.");
		final Locale locale;
		final UIViewRoot viewRoot = context.getViewRoot();
		if (context != null && viewRoot != null) {
			final Locale viewRootLocale = viewRoot.getLocale();
			if (viewRootLocale != null) {
				locale = viewRootLocale;
			} else {
				locale = Locale.getDefault();
			}
		} else {
			locale = Locale.getDefault();
		}
		final FacesMessage message = getMessage(severity, locale, messageId,
				params);
		return message;
	}

	public static FacesMessage getMessage(final Locale locale,
			final String messageId) {
		return getMessage(FacesMessage.SEVERITY_INFO, locale, messageId, null);
	}

	public static FacesMessage getMessage(final Locale locale,
			final String messageId, final Object... params) {
		return getMessage(FacesMessage.SEVERITY_INFO, locale, messageId, params);
	}

	public static FacesMessage getMessage(final FacesMessage.Severity severity,
			final Locale locale, final String messageId) {
		return getMessage(severity, locale, messageId, null);
	}

	public static FacesMessage getMessage(final FacesMessage.Severity severity,
			final Locale locale, final String messageId, final Object... params)
			throws MissingResourceException

	{
		Validate.notNull(severity, "Severity must not be null.");
		Validate.notNull(locale, "Locale must not be null.");
		Validate.notNull(messageId, "Message id must not be null.");
		final String bundleName = getApplication().getMessageBundle();
		Validate.notNull(bundleName, "Bundle name must be specified.");

		final ResourceBundle bundle = ResourceBundle.getBundle(bundleName,
				locale);

		final String summary = getText(messageId, bundle);

		String detail = null;
		try {
			final String detailText = bundle.getString(messageId
					+ DETAIL_SUFFIX);
			detail = substituteParams(locale, detailText, params);

		} catch (MissingResourceException ex) {
			// Nothing to do
		}

		if (detail == null) {
			return new FacesMessage(summary, summary);
		} else {
			return new FacesMessage(summary, detail);
		}
	}

	public static Application getApplication() {
		final FacesContext context = FacesContext.getCurrentInstance();
		if (context != null) {
			return FacesContext.getCurrentInstance().getApplication();
		} else {
			final ApplicationFactory afactory = (ApplicationFactory) FactoryFinder
					.getFactory(FactoryFinder.APPLICATION_FACTORY);
			return afactory.getApplication();
		}
	}

	public static String substituteParams(Locale locale, String msgtext,
			Object... params) {
		Validate.notNull(locale, "Locale must not be null.");
		Validate.notNull(msgtext, "Message text must not be null.");
		if (params == null) {
			return msgtext;
		} else {
			final MessageFormat format = new MessageFormat(msgtext);
			format.setLocale(locale);
			return format.format(params);
		}
	}

	/**
	 * Returns text for the given message id or
	 * <code>???<em>messageId</em>???</code> if resource bundle is null or
	 * entry could not be found.
	 * 
	 * @param messageId
	 *            message id.
	 * @param resourceBundle
	 *            resource bundle.
	 * @return Text for the given message id.
	 */
	public static String getText(String messageId, ResourceBundle resourceBundle) {
		Validate.notNull(messageId, "Message id must not be null.");
		if (resourceBundle != null) {
			try {
				final String text = resourceBundle.getString(messageId);
				return text;
			} catch (MissingResourceException ex) {
				return "???" + messageId + "???";
			}
		} else {
			return "???" + messageId + "???";
		}
	}
}
